/*  bvdd.h - bulverde(PXA27x) display driver
 */

#ifndef CHARDEV_H
#define CHARDEV_H

#include <linux/types.h>
#include <linux/ioctl.h>

#define BVDD_VERSION 0x000400	/* bvdd-0.4.0 */
#define BVDD_MAJOR_VERSION(v)	((v >> 16) & 0xFF)
#define BVDD_MINOR_VERSION(v)	((v >> 8) & 0xFF)
#define BVDD_PATCH_VERSION(v)	(v & 0xFF)

typedef struct {
    void *virt;			/* Virtual address */
    uint32_t phys;		/* Physical address */
} bvdd_addr_t;

typedef struct {
    int type;
    bvdd_addr_t addr;
    uint32_t size;
    uint32_t offset;		/* Internal uses only */
} bvdd_vram_area_t;

#define BVDD_VRAM_AREA_SRAM 0	/* PXA27x Internal SRAM */
#define BVDD_VRAM_AREA_PXAFB 1	/* Allocated by the SHARPSL_PXAFB */
#define BVDD_VRAM_AREA_BVDD 2	/* Allocated by the bvdd */
#define BVDD_MAX_VRAM_AREAS 3

#define BVDD_FORMAT_RGB2 1	/* 2-bits/pixel [4 entry palette] */
#define BVDD_FORMAT_RGB4 2	/* 4-bits/pixel [16 entry palette] */
#define BVDD_FORMAT_RGB8 3	/* 8-bits/pixel [256 entry palette] */
#define BVDD_FORMAT_RGB16 4	/* 16-bits/pixel */
#define BVDD_FORMAT_RGB18 5	/* 18-bits/pixel, unpacked */
#define BVDD_FORMAT_RGB18P 6	/* 18-bits/pixel, packed */
#define BVDD_FORMAT_RGB19 7	/* 19-bits/pixel, unpacked */
#define BVDD_FORMAT_RGB19P 8	/* 19-bits/pixel, packed */
#define BVDD_FORMAT_RGB24 9	/* 24-bits/pixel */
#define BVDD_FORMAT_RGB25 10	/* 25-bits/pixel */
#define BVDD_FORMAT_YUV444P 17	/* YCbCr4:4:4 Packed */
#define BVDD_FORMAT_YUV444 18	/* YCbCr4:4:4 Planar */
#define BVDD_FORMAT_YUV422 19	/* YCbCr4:2:2 Planar */
#define BVDD_FORMAT_YUV420 20	/* YCbCr4:2:0 Planar */

#define BVDD_FORMAT_YUV_FLAG 16

#define BVDD_FRAME_LAYER_BASE		0
#define BVDD_FRAME_LAYER_OVERLAY1	1
#define BVDD_FRAME_LAYER_OVERLAY2	2

typedef struct {
    uint8_t layer;		/* BVDD_FRAME_LAYER_BASE ... */
    uint8_t format;
    int8_t plane_id;
    int8_t plane_id_u;
    int8_t plane_id_v;
    uint16_t width;
    uint16_t height;
    uint16_t x;
    uint16_t y;
} bvdd_frame_t;

typedef struct {
    bvdd_addr_t addr;		/* Filled by bvdd */
    uint32_t size;
    int8_t area_type;
    int8_t id;			/* Filled by bvdd. descriptorsindex */
} bvdd_vram_plane_t;
#define BVDD_MAX_VRAM_PLANES 64

typedef struct {
    uint8_t r;
    uint8_t g;
    uint8_t b;
    uint8_t a;
} bvdd_color_t;

typedef struct {
    uint8_t layer;		/* BVDD_FRAME_LAYER_BASE ... */
    uint8_t start;
    uint16_t ncolors;
    bvdd_color_t *colors;
} bvdd_cmap_t;

/* The major device number. We can't rely on dynamic
 * registration any more, because ioctls need to know
 * it. */
#define BVDD_MAJOR_NUM 240

/* Set the message of the device driver */
#define BVDD_IOCTL_GET_VERSION _IOR(BVDD_MAJOR_NUM, 0, uint32_t)
#define BVDD_IOCTL_SET_FRAME _IOW(BVDD_MAJOR_NUM, 1, bvdd_frame_t *)
#define BVDD_IOCTL_BRANCH_FRAME _IOW(BVDD_MAJOR_NUM, 4, bvdd_frame_t *)
#define BVDD_IOCTL_DISABLE_OVERLAY _IOW(BVDD_MAJOR_NUM, 5, int)
#define BVDD_IOCTL_GET_VRAM_AREA_INFO _IOWR(BVDD_MAJOR_NUM, 7, bvdd_vram_area_t *)
#define BVDD_IOCTL_GET_VRAM_PLANE_INFO _IOWR(BVDD_MAJOR_NUM, 8, bvdd_vram_area_t *)
#define BVDD_IOCTL_ALLOC_PLANE _IOWR(BVDD_MAJOR_NUM, 9, bvdd_vram_plane_t *)
#define BVDD_IOCTL_SET_CMAP _IOW(BVDD_MAJOR_NUM, 10, bvdd_cmap_t *)
#define BVDD_IOCTL_DEBUG _IO(BVDD_MAJOR_NUM, 64)
#define BVDD_IOCTL_SET_IWMMXT_STATE _IOW(BVDD_MAJOR_NUM, 65, int)

/* The name of the device file */
#define BVDD_DEVICE_FILE_NAME "bvdd"

#endif
