#ifndef BVDD_P_INCLUDED
#define BVDD_P_INCLUDED

#ifdef DEBUG
# define PRINT_DEBUG(x...) printk(KERN_WARNING DEVICE_NAME ": " x)
# define PRINT_INFO(x...) printk(KERN_WARNING DEVICE_NAME ": " x)
# define PRINT_WARN(x...) printk(KERN_WARNING DEVICE_NAME ": " x)
# define PRINT_ERROR(x...) printk(KERN_ERR DEVICE_NAME ": " x)
# define PRINT_FATAL(x...) printk(KERN_CRIT DEVICE_NAME ": " x)
#else
# define PRINT_DEBUG(x...)
# define PRINT_INFO(x...)  printk(KERN_WARNING DEVICE_NAME ": " x)
# define PRINT_WARN(x...) printk(KERN_WARNING DEVICE_NAME ": " x)
# define PRINT_ERROR(x...) printk(KERN_ERR DEVICE_NAME ": " x)
# define PRINT_FATAL(x...) printk(KERN_CRIT DEVICE_NAME ": " x)
#endif

#define DEVICE_NAME "bvdd"	/* The name for our device */

typedef struct {
    uint32_t FDADR;	/* frame descriptor address */
    uint32_t FSADR;      /* DMA Frame Source Address Registers */
    uint32_t FIDR;		/* frame id reg */
    uint32_t LDCMD;	/* LCD command reg */
} descriptor_t;

#ifndef USE_PXAFB_DESCRIPTOR_AREA
# define DESCRIPTORS_SIZE (PAGE_SIZE)
# define MAX_DESCRIPTORS (DESCRIPTORS_SIZE / 16)
#else
# define DESCRIPTORS_SIZE (PAGE_SIZE - 16)
# define MAX_DESCRIPTORS (DESCRIPTORS_SIZE / 16)
#endif
#define DESCRIPTOR_ID_FOR_E25 (MAX_DESCRIPTORS - 1)
#define DESCRIPTOR_ID_FOR_PALETTE (MAX_DESCRIPTORS - 4)
#define AVAILABLE_DESCRIPTORS (MAX_DESCRIPTORS - 5)

#define LAYER_BASE 0
#define LAYER_OVERLAY1 1
#define LAYER_OVERLAY2 2

#define TIMEOUT 30000

#define SRAM_BASE_PHYS 0x5C000000UL
#define SRAM_BASE_VIRT ((void *)0xFE100000UL)

#include <asm/hardware.h>
#undef FDADR

#define REG_LCCR0	__REG(0x44000000) /* LCD Controller Control register 0 */
#define REG_LCCR1	__REG(0x44000004) /* LCD Controller Control register 1 */
#define REG_LCCR2	__REG(0x44000008) /* LCD Controller Control register 2 */
#define REG_LCCR3	__REG(0x4400000C) /* LCD Controller Control register 3 */
#define REG_LCCR4	__REG(0x44000010) /* LCD Controller Control register 4 */
#define REG_LCCR5	__REG(0x44000014) /* LCD Controller Control register 5 */
#define REG_FBR0	__REG(0x44000020)
#define REG_FBR1	__REG(0x44000024)
#define REG_FBR2	__REG(0x44000028)
#define REG_FBR3	__REG(0x4400002C)
#define REG_FBR4	__REG(0x44000030)
#define REG_LCSR1	__REG(0x44000034)
#define REG_LCSR0	__REG(0x44000038)
#define REG_LIIDR	__REG(0x4400003C)
#define REG_TRGBR	__REG(0x44000040)
#define REG_TCR		__REG(0x44000044)
#define REG_OVL1C1	__REG(0x44000050)
#define REG_OVL1C2	__REG(0x44000060)
#define REG_OVL2C1	__REG(0x44000070)
#define REG_OVL2C2	__REG(0x44000080)
#define REG_CCR		__REG(0x44000090)
#define REG_CMDCR	__REG(0x44000100)
#define REG_PRSR	__REG(0x44000104)
#define REG_FBR5	__REG(0x44000110)
#define REG_FBR6	__REG(0x44000114)
#define REG_FDADR0	__REG(0x44000200)
#define REG_FSADR0	__REG(0x44000204)
#define REG_FIDR0	__REG(0x44000208)
#define REG_LDCMD0	__REG(0x4400020C)
#define REG_FDADR1	__REG(0x44000210)
#define REG_FSADR1	__REG(0x44000214)
#define REG_FIDR1	__REG(0x44000218)
#define REG_LDCMD1	__REG(0x4400021C)
#define REG_FDADR2	__REG(0x44000220)
#define REG_FSADR2	__REG(0x44000224)
#define REG_FIDR2	__REG(0x44000228)
#define REG_LDCMD2	__REG(0x4400022C)
#define REG_FDADR3	__REG(0x44000230)
#define REG_FSADR3	__REG(0x44000234)
#define REG_FIDR3	__REG(0x44000238)
#define REG_LDCMD3	__REG(0x4400023C)
#define REG_FDADR4	__REG(0x44000240)
#define REG_FSADR4	__REG(0x44000244)
#define REG_FIDR4	__REG(0x44000248)
#define REG_LDCMD4	__REG(0x4400024C)
#define REG_FDADR5	__REG(0x44000250)
#define REG_FSADR5	__REG(0x44000254)
#define REG_FIDR5	__REG(0x44000258)
#define REG_LDCMD5	__REG(0x4400025C)
#define REG_FDADR6	__REG(0x44000260)
#define REG_FSADR6	__REG(0x44000264)
#define REG_FIDR6	__REG(0x44000268)
#define REG_LDCMD6	__REG(0x4400026C)
#define REG_LCDBSCNTR	__REG(0x48000054)

typedef struct {
    uint32_t ENB		: 1; /* LCD Controller Enable */
    uint32_t CMS		: 1; /* Color/Monochrome Select */
    uint32_t SDS		: 1; /* Single-Scan/dual-scan Display Select */
    uint32_t LDM		: 1; /* LCD Disable Done Mask */
    uint32_t SOFM0		: 1; /* Start of Frame Mask for Channel 0 and Channel 1 (Dual Scan) */
    uint32_t IUM		: 1; /* Input FIFO Underrun Mask */
    uint32_t EOFM0		: 1; /* End of Frame Mask for Channel 0 and for Channel 1 (Dual Scan) */
    uint32_t PAS		: 1; /* Passive/Active Display Select */
    uint32_t reserved2	: 1;
    uint32_t DPD		: 1; /* Double-Pixel Data (DPD) Pin Mode */
    uint32_t DIS		: 1; /* LCD Disable */
    uint32_t QDM		: 1; /* LCD Quick Disable Mask */
    uint32_t PDD		: 8; /* Palette DMA Request Delay */
    uint32_t BSM0		: 1; /* Branch Status Mask */
    uint32_t OUM		: 1; /* Output FIFO Underrun Mask */
    uint32_t LCDT		: 1; /* LCD Panel Type */
    uint32_t RDSTM		: 1; /* LCD Read Status Interrupt Mask */
    uint32_t CMDIM		: 1; /* LCD Command Interrupt Mask */
    uint32_t OUC		: 1; /* Overlay Underlay Control Bit */
    uint32_t LDDALT	: 1; /* LDD Alternate Mapping Control Bit */
    uint32_t reserved1     : 5;
} LCCR0_t;
typedef union {
    LCCR0_t f;
    uint32_t val;
} LCCR0_u;

typedef struct {
    uint32_t PPL		:10; /* Pixels per Line for the Base Frame */
    uint32_t HSW		: 6; /* Horizontal Sync Pulse Width */
    uint32_t ELW		: 8; /* End-of-Line Pixel Clock Wait Count */
    uint32_t BLW		: 8; /* Beginning-of-Line Pixel Clock Wait Count */
} LCCR1_t;
typedef union {
    LCCR1_t f;
    uint32_t val;
} LCCR1_u;

typedef struct {
    uint32_t LPP		:10; /* Lines per Panel for the Base Frame */
    uint32_t VSW		: 6; /* Vertical Sync Pulse Width */
    uint32_t EFW		: 8; /* End-of-Frame Line Clock Wait Count */
    uint32_t BFW		: 8; /* Beginning-of-Frame Line Clock Wait Count */
} LCCR2_t;
typedef union {
    LCCR2_t f;
    uint32_t val;
} LCCR2_u;

typedef struct {
    uint32_t PCD		: 8; /* Pixel Clock Divisor */
    uint32_t ACB		: 8; /* AC Bias Pin Frequency */
    uint32_t API		: 4; /* AC Bias Pin Transitions per Interrupt */
    uint32_t VSP		: 1; /* Vertical Sync Polarity */
    uint32_t HSP		: 1; /* Horizontal Sync Polarity */
    uint32_t PCP		: 1; /* Pixel Clock Polarity */
    uint32_t OEP		: 1; /* Output Enable Polarity */
    uint32_t BPP		: 3; /* Bits per Pixel */
    uint32_t DPC		: 1; /* Double Pixel Clock Mode */
    uint32_t reserved1	: 1;
    uint32_t BPP3		: 1; /* Bits per Pixel */
    uint32_t PDFOR		: 2; /* Pixel Data Format */
} LCCR3_t;
typedef union {
    LCCR3_t f;
    uint32_t val;
} LCCR3_u;

typedef struct {
    uint32_t K1		: 3; /* Multiplication Constant for Red for Half Transparency */
    uint32_t K2		: 3; /* Multiplication Constant for Blue for Half Transparency */
    uint32_t K3		: 3; /* Multiplication Constant for Green for Half Transparency */
    uint32_t reserved2	: 6;
    uint32_t PAL_FOR	: 2; /* Palette Data Format */
    uint32_t reserved1	:12;
    uint32_t PCDDIV	: 1; /* PCD Divisor Selection */
} LCCR4_t;
typedef union {
    LCCR4_t f;
    uint32_t val;
} LCCR4_u;

/* LCCR5 unused */

typedef struct {
    uint32_t PPL1		:10; /* Pixels per Line for Overlay 1 Frame */
    uint32_t LPO1		:10; /* Number of Lines for Overlay 1 */
    uint32_t BPP1		: 4; /* Bits per Pixel for Overlay 1 */
    uint32_t reserved1	: 7;
    uint32_t O1EN		: 1; /* Enable bit for Overlay 1 */
} OVL1C1_t;
typedef union {
    OVL1C1_t f;
    uint32_t val;
} OVL1C1_u;

typedef struct {
    uint32_t O1XPOS	:10; /* Horizontal Position of the Upper Left-Most Pixel of Overlay 1 Window */
    uint32_t O1YPOS	:10; /* Vertical Position of the Upper Left-Most Pixel of Overlay 1 Window */
    uint32_t reserved1	:12;
} OVL1C2_t;
typedef union {
    OVL1C2_t f;
    uint32_t val;
} OVL1C2_u;

typedef struct {
    uint32_t PPL2		:10; /* Pixels per Line for Overlay 2 Frame */
    uint32_t LPO2		:10; /* Number of Lines for Overlay 2 Frame */
    uint32_t BPP2		: 4; /* Bits per Pixel for Overlay 2 */
    uint32_t reserved1	: 7;
    uint32_t O2EN		: 1; /* Overlay 2 Enable */
} OVL2C1_t;
typedef union {
    OVL2C1_t f;
    uint32_t val;
} OVL2C1_u;

typedef struct {
    uint32_t O2XPOS	:10; /* Horizontal Position of the Upper Left Most Pixel of Overlay 2 */
    uint32_t O2YPOS	:10; /* Vertical Position of the Upper Left Most Pixel of Overlay 2 */
    uint32_t FOR		: 3; /* Format */
    uint32_t reserved1	: 9;
} OVL2C2_t;
typedef union {
    OVL2C2_t f;
    uint32_t val;
} OVL2C2_u;

#undef BS0
typedef struct {
    uint32_t LDD		: 1; /* LCD Disable Done Flag */
    uint32_t SOF0		: 1; /* Start of Frame Status for Base */
    uint32_t BER		: 1; /* Bus Error Status */
    uint32_t ABC		: 1; /* AC Bias Count Status */
    uint32_t IU0		: 1; /* Input FIFO Underrun for Channel 0 */
    uint32_t IU1		: 1; /* Input FIFO Underrun for Channel 1 */
    uint32_t OU		: 1; /* Output FIFO Underrun */
    uint32_t QD		: 1; /* LCD Quick Disable Status */
    uint32_t EOF0		: 1; /* End of Frame Status for Base (Channel 0) */
    uint32_t BS0		: 1; /* Branch Status for Base */
    uint32_t SINT		: 1; /* Subsequent Interrupt Status */
    uint32_t RD_ST		: 1; /* Read Status */
    uint32_t CMD_INT	: 1; /* Command Interrupt Status */
    uint32_t reserved2	:15;
    uint32_t BER_CH	: 3; /* Bus Error Channel Number */
    uint32_t reserved1	: 1;
} LCSR0_t;
typedef union {
    LCSR0_t f;
    uint32_t val;
} LCSR0_u;

#undef BS1
typedef struct {
    uint32_t SOF1		: 1; /* Start of Frame Status for Channel 1 */
    uint32_t SOF2		: 1; /* Start of Frame Status for Channel 2 */
    uint32_t SOF3		: 1; /* Start of Frame Status for Channel 3 */
    uint32_t SOF4		: 1; /* Start of Frame Status for Channel 4 */
    uint32_t SOF5		: 1; /* Start of Frame Status for Channel 5 */
    uint32_t SOF6		: 1; /* Start of Frame Status for Channel 6 */
    uint32_t reserved4	: 2;
    uint32_t EOF1		: 1; /* End of Frame Status for Channel 1 */
    uint32_t EOF2		: 1; /* End of Frame Status for Channel 2 */
    uint32_t EOF3		: 1; /* End of Frame Status for Channel 3 */
    uint32_t EOF4		: 1; /* End of Frame Status for Channel 4 */
    uint32_t EOF5		: 1; /* End of Frame Status for Channel 5 */
    uint32_t EOF6		: 1; /* End of Frame Status for Channel 6 */
    uint32_t reserved3	: 2;
    uint32_t BS1		: 1; /* Branch Status for Channel 1 (Command Register) */
    uint32_t BS2		: 1; /* Branch Status for Channel 2 (Command Register) */
    uint32_t BS3		: 1; /* Branch Status for Channel 3 (Command Register) */
    uint32_t BS4		: 1; /* Branch Status for Channel 4 (Command Register) */
    uint32_t BS5		: 1; /* Branch Status for Channel 5 (Command Register) */
    uint32_t BS6		: 1; /* Branch Status for Channel 6 (Command Register) */
    uint32_t reserved2	: 3;
    uint32_t IU2		: 1; /* Input FIFO Underrun for Channel 2 */
    uint32_t IU3		: 1; /* Input FIFO Underrun for Channel 3 */
    uint32_t IU4		: 1; /* Input FIFO Underrun for Channel 4 */
    uint32_t IU5		: 1; /* Input FIFO Underrun for Channel 5 */
    uint32_t IU6		: 1; /* Input FIFO Underrun for Channel 6 */
    uint32_t reserved1	: 2;
} LCSR1_t;
typedef union {
    LCSR1_t f;
    uint32_t val;
} LCSR1_u;

/* CCR unused */
/* CMDCR unused */
/* TRGBR unused */
/* TCR unused */

typedef struct {
    uint32_t LCDBS		: 4; /* LCD Buffer Strength */
    uint32_t reserved1	:28;
} LCDBSCNTR_t;
typedef union {
    LCDBSCNTR_t f;
    uint32_t val;
} LCDBSCNTR_u;

typedef struct {
    LCCR0_u lccr0;
    LCCR1_u lccr1;
    LCCR2_u lccr2;
    LCCR3_u lccr3;
    LCCR4_u lccr4;
    OVL1C1_u ovl1c1;
    OVL1C2_u ovl1c2;
    OVL2C1_u ovl2c1;
    OVL2C2_u ovl2c2;
    uint32_t fdadr0;
    uint32_t fdadr1;
    uint32_t fdadr2;
    uint32_t fdadr3;
    uint32_t fdadr4;
    uint32_t fdadr5;
    uint32_t fdadr6;
    char cken20_intmem;
} lcdc_state_t;

#endif /* BVDD_P_INCLUDED */
