/*
 * idct_arm_ipp.c
 * Copyright (C) 2004 AGAWA Koji <i (AT) atty (DOT) jp>
 *
 * This file is part of mpeg2dec, a free MPEG-2 video stream decoder.
 * See http://libmpeg2.sourceforge.net/ for updates.
 *
 * mpeg2dec is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * mpeg2dec is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "config.h"

#ifdef ARCH_ARM
#ifdef HAVE_IPP

#include <string.h>
#include <inttypes.h>
#include "ipp.h"

#include "mpeg2.h"
#include "attributes.h"
#include "mpeg2_internal.h"

static int add_pixels_clamped_ARM(short *block, unsigned char *dest, int line_size)
{
    asm volatile (
		  //"pld [%1] \n\t"
		  //"pld [%0] \n\t"
		  "sub sp, sp, #8 \n\t" /* allow 2 local variables */
		  "str %0, [sp, #0] \n\t"
		  "str %1, [sp, #4] \n\t"
		  "mov r10, #8 \n\t"

		  "1: \n\t"

		  /* load dest */
		  "ldr r4, [%1] \n\t"
		  /* block[0] and block[1]*/
		  "ldrsh r5, [%0] \n\t"
		  "ldrsh r7, [%0, #2] \n\t"
		  "and r6, r4, #0xFF \n\t"
		  "and r8, r4, #0xFF00 \n\t"
		  "add r6, r5, r6 \n\t"
		  "add r8, r7, r8, lsr #8 \n\t"
		  "mvn r5, r5 \n\t"
		  "mvn r7, r7 \n\t"
		  "tst r6, #0x100 \n\t"
		  "movne r6, r5, lsr #24 \n\t"
		  "tst r8, #0x100 \n\t"
		  "movne r8, r7, lsr #24 \n\t"
		  "mov r9, r6 \n\t"
		  "ldrsh r5, [%0, #4] \n\t" /* moved form [A] */
		  "orr r9, r9, r8, lsl #8 \n\t"
		  /* block[2] and block[3] */
		  /* [A] */
		  "ldrsh r7, [%0, #6] \n\t"
		  "and r6, r4, #0xFF0000 \n\t"
		  "and r8, r4, #0xFF000000 \n\t"
		  "add r6, r5, r6, lsr #16 \n\t"
		  "add r8, r7, r8, lsr #24 \n\t"
		  "mvn r5, r5 \n\t"
		  "mvn r7, r7 \n\t"
		  "tst r6, #0x100 \n\t"
		  "movne r6, r5, lsr #24 \n\t"
		  "tst r8, #0x100 \n\t"
		  "movne r8, r7, lsr #24 \n\t"
		  "orr r9, r9, r6, lsl #16 \n\t"
		  "ldr r4, [%1, #4] \n\t"	/* moved form [B] */
		  "orr r9, r9, r8, lsl #24 \n\t"
		  /* store dest */
		  "ldrsh r5, [%0, #8] \n\t" /* moved form [C] */
		  "str r9, [%1] \n\t"

		  /* load dest */
		  /* [B] */
		  /* block[4] and block[5] */
		  /* [C] */
		  "ldrsh r7, [%0, #10] \n\t"
		  "and r6, r4, #0xFF \n\t"
		  "and r8, r4, #0xFF00 \n\t"
		  "add r6, r5, r6 \n\t"
		  "add r8, r7, r8, lsr #8 \n\t"
		  "mvn r5, r5 \n\t"
		  "mvn r7, r7 \n\t"
		  "tst r6, #0x100 \n\t"
		  "movne r6, r5, lsr #24 \n\t"
		  "tst r8, #0x100 \n\t"
		  "movne r8, r7, lsr #24 \n\t"
		  "mov r9, r6 \n\t"
		  "ldrsh r5, [%0, #12] \n\t" /* moved from [D] */
		  "orr r9, r9, r8, lsl #8 \n\t"
		  /* block[6] and block[7] */
		  /* [D] */
		  "ldrsh r7, [%0, #14] \n\t"
		  //"pld [%0], #16] \n\t"
		  "and r6, r4, #0xFF0000 \n\t"
		  "and r8, r4, #0xFF000000 \n\t"
		  "add r6, r5, r6, lsr #16 \n\t"
		  "add r8, r7, r8, lsr #24 \n\t"
		  "mvn r5, r5 \n\t"
		  "mvn r7, r7 \n\t"
		  "tst r6, #0x100 \n\t"
		  "movne r6, r5, lsr #24 \n\t"
		  "tst r8, #0x100 \n\t"
		  "movne r8, r7, lsr #24 \n\t"
		  "orr r9, r9, r6, lsl #16 \n\t"
		  "add %0, %0, #16 \n\t" /* moved from [E] */
		  "orr r9, r9, r8, lsl #24 \n\t"
		  "subs r10, r10, #1 \n\t" /* moved from [F] */
		  /* store dest */
		  "str r9, [%1, #4] \n\t"

		  /* [E] */
		  /* [F] */
		  "add %1, %1, %2 \n\t"
		  "bne 1b \n\t"

		  "ldr %0, [sp, #0] \n\t"
		  "ldr %1, [sp, #4] \n\t"
		  "add sp, sp, #8 \n\t"
		  :
		  : "r"(block),
		    "r"(dest),
		    "r"(line_size)
		  : "r4", "r5", "r6", "r7", "r8", "r9", "r10", "cc", "memory" );
}

void mpeg2_idct_copy_arm_ipp (int16_t * block, uint8_t * dest, int stride)
{
/*     ippiDCTInv_8x8_16s8u(block, dest, stride); */
    ippiDCT8x8Inv_Video_16s8u_C1R(block, dest, stride);
    memset (block, 0, 64 * sizeof (uint16_t));
}

void mpeg2_idct_add_arm_ipp (int last, int16_t * block,
			     uint8_t * dest, int stride)
{
/*     ippiDCTInv_8x8_16s_I(block); */
    ippiDCT8x8Inv_Video_16s_C1I(block);
    add_pixels_clamped_ARM(block, dest, stride);
    memset (block, 0, 64 * sizeof (uint16_t));
}

#endif
#endif
